/*
   Affichages de triangles.
   Illustration des procdures rcursives et des boucles.
*/

import java.awt.*;
import java.applet.*;

public class Triangles extends Applet {

  private void drawTriangle ( Graphics g,
                              int x, int y,
                              int width, int height ) {

    // Trace un triangle isocle inscrit dans un rectangle,
    // la base du triangle est celle du rectangle et le sommet
    // est le centre du haut du rectangle.
    // On dtermine les trois sommets et on trace une ligne
    // entre eux.

    if( height >= 0 && width >= 0 ) {
      int x1 = x + width/2,
          y1 = y,
          x2 = x,
          y2 = y + height,
          x3 = x + width,
          y3 = y + height;
      g.drawLine( x1, y1, x2, y2);
      g.drawLine( x2, y2, x3, y3);
      g.drawLine( x3, y3, x1, y1);
    }
  }

  private void fillTriangle ( Graphics g,
                              int x, int y,
                              int width, int height ) {

    if( width > 0 && height > 0 ) {
      drawTriangle( g, x, y, width, height );
      fillTriangle( g, x+1, y+1, width - 2, height - 2 );
    } else if( width > 0 ) g.drawLine( x, y, x + width, y );
    else if( height > 0 ) g.drawLine( x, y, x, y + height );
  }

  private void fillTriangleRec( Graphics g,
                                float x, int y,
                                float width, int height){

    // Remplit un triangle en traant la base en se rappelant
    // rcursivement pour tracer le triangle au-dessus de cette
    // ligne.  Attention x et width sont des float pour viter
    // les accumulations d'erreur de troncatures ou d'arrondi...

    g.drawLine( Math.round(x),
                y + height,
                Math.round( x + width ),
                y + height);

    if( height > 1) {
      int height1 = height - 1; // nouvelle hauteur
      // largeur et dcalage en x calculs par la loi
      // des triangles semblables.
      float width1 = width * height1/height;
      float dx = ( width - width1 ) / 2;
      fillTriangleRec( g, x + dx, y, width1, height1 );
    }
  }

  private void iterativeFillTriangle( Graphics g,
                                      int x, int y,
                                      int width, int height ) {

    // Remplit un triangle de faon itrative.
    // On recalcule  chaque ligne la valeur
    // exacte du point de dpart.

    float ratio = (float)width / height;
    for( int dy = 0; dy <= height; dy++) {
      float width1 = dy * ratio;
      float dx = ( width - width1 ) / 2;
      g.drawLine( Math.round( x + dx ),
                  y+dy,
                  Math.round( x + dx + width1),
                  y + dy);
    }
  }

  private float surface( int width, int height ) {
    return ( width * height ) / 2.0f;
  }

  private void drawTriangleSurface( Graphics g,
                                    int x, int y,
                                    int width, int height) {

    drawTriangle( g, x, y, width, height );
    g.drawString( Float.toString( surface( width, height )),
                  x + width / 2,
                  y + height / 2);
  }

  private void testFillTriangle( Graphics g,
                                 int x, int y,
                                 int width, int height ) {

    g.setColor( Color.green );
    iterativeFillTriangle( g, x, y, width, height );
    g.setColor( Color.red );
    drawTriangle( g, x, y, width, height );
  }

  public void paint( Graphics g ) {
    testFillTriangle( g, 10, 0, 200, 150 );
    testFillTriangle( g, 100, 175, 20, 150 );
  }
}
